import processing.core.*; 
import processing.xml.*; 

import traer.physics.*; 

import java.applet.*; 
import java.awt.Dimension; 
import java.awt.Frame; 
import java.awt.event.MouseEvent; 
import java.awt.event.KeyEvent; 
import java.awt.event.FocusEvent; 
import java.awt.Image; 
import java.io.*; 
import java.net.*; 
import java.text.*; 
import java.util.*; 
import java.util.zip.*; 
import java.util.regex.*; 

public class Distancia_Interatomica extends PApplet {


/* 

   Nesta anima\u00e7\u00e3o foi utilizada a biblioteca 'physics' desenvolvida na Universidade de Princeton, EUA, a
   biblioteca consiste na uni\u00e3o de classes e m\u00e9todos de simula\u00e7\u00e3o   de intera\u00e7\u00f5es f\u00edsicas, com a possibilidade
   de estabelecimento de for\u00e7as de atra\u00e7\u00e3o e repuls\u00e3o, assim como for\u00e7as gravitacionais em objetos de uma 
   classe de part\u00edculas.
   
   Mais informa\u00e7\u00f5es a respeito da biblioteca em: http://www.cs.princeton.edu/~traer/physics/ 
   
   */


ParticleSystem physics; // sistema de part\u00edculas
Particle a, b; // \u00e1tomos
Spring s; // for\u00e7a entre os \u00e1tomos
//Attraction at;

Grafico g; // gr\u00e1fico
  PVector origem; // auxiliar de posicionamento
Botao[] grafico; //bot\u00f5es de escolha do gr\u00e1fico apresentado
Credito credito;

PFont f; //fonte

//CONSTANTES
float raioA = 20; // raio do \u00e1tomo que se move
float raioB = 40; // raio do \u00e1tomo fixo

boolean modo = true; //quando modo = TRUE, mostramos o gr\u00e1fico da energia
                     //quando modo = FALSE, mostramos o gr\u00e1fico da for\u00e7a

public void setup(){
  size(500, 600/*, P3D*/);
  smooth();
  noStroke();
  rectMode(CORNERS);
  textAlign(CENTER);
    
  //inicializa\u00e7\u00e3o
  physics = new ParticleSystem(0, .1f);
  a = physics.makeParticle(1, 3*width/4, 5*height/16, 0);
  b = physics.makeParticle(5, width/4, a.position().y(), 0);
  b.makeFixed();
  
  // inicializa\u00e7\u00e3o do gr\u00e1fico 
  origem = new PVector(b.position().x(), b.position().y() + 3*raioB );
  g = new Grafico(origem, 100, 250, 30, 0, 100, -250);  
  
  g.rmin = (raioA + raioB)/g.escala_x; // colocamos o ponto mais baixo do grafico de energia na posi\u00e7\u00e3o certa (soma dos raios)
  
  // inicializa\u00e7\u00e3o das for\u00e7as
  s = physics.makeSpring(a, b, .2f, .3f, /*115*/g.rmin*g.escala_x);
  //at = physics.makeAttraction(a, b, -10000, 500);
  
  //inicializa\u00e7\u00e3o dos bot\u00f5es
  grafico = new Botao[2];
  grafico[0] = new Botao(8*width/11, origem.y + height/8, width/10, 12, "Energia");
    grafico[0].situacao = "SELECIONADO";
    grafico[0].psituacao = "LIBERADO";
  grafico[1] = new Botao(grafico[0].xpos + 1.4f*grafico[0].comprimento, origem.y + height/8, 3*width/40, 12, "For\u00e7a");
  
  //inicializa\u00e7\u00e3o do bot\u00e3o de cr\u00e9ditos
  credito = new Credito();
  
  // inicializa\u00e7\u00e3o da fonte
  f = loadFont("CourierNewPS-BoldMT-28.vlw");
  textFont(f, 38);
  
  
  background(255);  
  
  // texto
  fill(0);
  text("Dist\u00e2ncia de Liga\u00e7\u00e3o", width/2, height/8);
  textFont(f, 12);
  text("Gr\u00e1fico:", 6*width/11, origem.y + height/8);
  textFont(f, 14);
  text("A dist\u00e2ncia de liga\u00e7\u00e3o de dois \u00e1tomos \u00e9 a dist\u00e2ncia\n" +
       "correspondente ao ponto de m\u00ednima energia potencial,\n" +
       "no qual a for\u00e7a resultante entre os \u00e1tomos assume\n" +
       "valor nulo devido \u00e0 igualdade dos m\u00f3dulos e oposi\u00e7\u00e3o\n" +
       "dos sentidos das for\u00e7as atrativas e repulsivas.\n\n" +
       "Arraste o \u00e1tomo para outra posi\u00e7\u00e3o e observe como\n" + 
       "o mesmo assume naturalmente a dist\u00e2ncia de liga\u00e7\u00e3o,\n" +
       "procurando a situa\u00e7\u00e3o de maior estabilidade.\n", width/2, 3*height/4);         
}

public void draw(){
  
  if(credito.botao.situacao != "LIBERADO"){
    textFont(f, 38);
    background(255);
    fill(0);
    text("Dist\u00e2ncia de Liga\u00e7\u00e3o", width/2, height/8);
    textFont(f, 12);
    text("Gr\u00e1fico:", 6*width/11, origem.y + height/8);
    textFont(f, 14);
    text("A dist\u00e2ncia de liga\u00e7\u00e3o de dois \u00e1tomos \u00e9 a dist\u00e2ncia\n" +
         "correspondente ao ponto de m\u00ednima energia potencial,\n" +
         "no qual a for\u00e7a resultante entre os \u00e1tomos assume\n" +
         "valor nulo devido \u00e0 igualdade dos m\u00f3dulos e oposi\u00e7\u00e3o\n" +
         "dos sentidos das for\u00e7as atrativas e repulsivas.\n\n" +
         "Arraste o \u00e1tomo para outra posi\u00e7\u00e3o e observe como\n" + 
         "o mesmo assume naturalmente a dist\u00e2ncia de liga\u00e7\u00e3o,\n" +
         "procurando a situa\u00e7\u00e3o de maior estabilidade.\n", width/2, 3*height/4);     
  }
  
  fill(255);
  rectMode(CORNERS);
  rect(0, b.position().y() - raioB, width, b.position().y() + 3*raioB + g.y_max*g.escala_y + 5); //ao inv\u00e9s de atualizarmos a tela inteira
  physics.tick(.2f);                                                                              //o fazemos apenas com o ret\u00e2ngulo em que
  display();                                                                                     //a anima\u00e7\u00e3o ocorre
  g.desenha_grafico();
  segue();
  botaoDisplay();
  credito.display();
}

// fun\u00e7\u00e3o que desenha os \u00e1tomos 
public void display(){
  pushMatrix();
  translate(b.position().x(), b.position().y()/*, b.position().z()*/);
  fill(162, 205, 255);
  //sphere(40);
  ellipse(0, 0, 2*raioB, 2*raioB);
  fill(100);
  ellipse(0, 0, 10, 10);
  popMatrix();
  pushMatrix();
  translate(a.position().x(), a.position().y()/*, a.position().z()*/);
  fill(247, 175, 255);
  //sphere(20);
  ellipse(0, 0, 2*raioA, 2*raioA);
  fill(100);
  ellipse(0, 0, 10, 10);
  popMatrix();
}

// fun\u00e7\u00e3o que desenha os bot\u00f5es
public void botaoDisplay(){
  rectMode(CENTER);
  textFont(f, 11);
  grafico[0].display();
  grafico[1].display();
  rectMode(CORNERS);
  textFont(f, 12);
}
// fun\u00e7\u00e3o que desenha a bolinha que acompanha o gr\u00e1fico
public void segue(){
 float dx = a.position().x() - g.o.x;
 float t = dx/(g.escala_x);
 float p = (g.rmin)/t;
 float yt = g.funcao(p)/(g.escala_y);
 if((t < g.x_max - 1 && t > g.x_min) && (yt <= g.y_max && yt > g.y_min)){
   noStroke();
   fill(255, 0, 0);
   ellipse(g.o.x + t*(g.escala_x), g.o.y + yt*(g.escala_y), 10, 10);
 }
}
  
public void mousePressed(){
  
  /*  
  //o bot\u00e3o direito do mouse determina qual o gr\u00e1fico apresentado
  if(mouseButton == RIGHT)
    modo = !modo;
  */
  
  // com o bot\u00e3o esquerdo do mouse, deslocamos o \u00e1tomo livre por seu eixo x  
  if(mouseButton == LEFT){
    grafico[0].mousePressed();
    grafico[1].mousePressed();
    
    float dx = mouseX - a.position().x();
    float dy = mouseY - a.position().y();
    float d = sqrt(dx*dx + dy*dy);
  
    if( d <= 20 && mouseX - b.position().x() > raioA + raioB && mouseX < width - raioA ){
      a.makeFixed(); // para que o \u00e1tomo n\u00e3o se mova quando paramos o mouse, fixamos-lo
      a.position().set(mouseX,  a.position().y(), 0); // o \u00e1tomo livre desloca-se para a posi\u00e7\u00e3o do mouse
    }
  }
  credito.botao.mousePressed();
}

public void mouseDragged(){
  if(mouseButton == LEFT){
    float dx = mouseX - a.position().x();
    float dy = mouseY - a.position().y();
    float d = sqrt(dx*dx + dy*dy);
  
    if( d <= 20 && mouseX - b.position().x() > raioA /*+ raioB*/ && mouseX < width - raioB ){
      a.makeFixed(); // para que o \u00e1tomo n\u00e3o se mova quando paramos o mouse, fixamos-lo
      a.position().set(mouseX,  a.position().y(), 0); // o \u00e1tomo livre desloca-se para a posi\u00e7\u00e3o do mouse
    }
    else
      a.makeFree(); // se o \u00e1tomo n\u00e3o \u00e9 selecionado, ele est\u00e1 livre
  }
}

public void mouseReleased(){
  /*
  for(int i = 0; i < grafico.length; i++)
    if(grafico[i].situacao == "PRESSIONADO"){
      for(int j = 0; j < grafico.length; j++)
        if(j != i)
          grafico[j].libera();
      grafico[i].mouseReleased();
      modo = !modo;
    }
  */
  
   if(grafico[0].situacao == "PRESSIONADO"){
     modo = true;
     grafico[1].libera();
     grafico[0].mouseReleased();
   }
   
   if(grafico[1].situacao == "PRESSIONADO"){
     modo = false;
     grafico[0].libera();
     grafico[1].mouseReleased();
    }
    
  a.makeFree(); // quando liberamos o bot\u00e3o, o \u00e1tomo passa a sofrer as for\u00e7as interat\u00f4micas
  
  credito.botao.mouseReleased();
}
// BOT\u00c3O///

class Botao{
  float xpos, ypos; // coordenadas do centro do bot\u00e3o
  
  String texto, // guarda o texto a ser impresso no bot\u00e3o
  
         situacao, // guarda a situa\u00e7\u00e3o em que se encontra o bot\u00e3o, podendo estar "LIBERADO", "PRESSIONADO" ou "SELECIONADO"
  
         psituacao;  // guarda a situa\u00e7\u00e3o 'est\u00e1vel' que o bot\u00e3o assumira antes de ser pressionado pela \u00faltima vez
                     // (pode guardar "LIBERADO" ou "SELECIONADO") 
  
  float comprimento; // comprimento de refer\u00eancia do bot\u00e3o (ret\u00e2ngulo interno)
  
  float altura; // altura de refer\u00eancia do bot\u00e3o (ret\u00e2ngulo interno)
  
  
  //CONSTRUTORA SIMPLES//
  Botao(float _xpos, float _ypos, String _texto){
    xpos = _xpos;
    ypos = _ypos;
    texto = _texto;
    situacao = "LIBERADO";        // a priore, o bot\u00e3o \u00e9 iniciado como se n\u00e3o estivesse ativado 
    psituacao = "SELECIONADO";    // caso queiramos que ele apresente-se 'selecionado' logo de
                                  // in\u00edcio, trocamos estas vari\u00e1veis apos a chamada da fun\u00e7\u00e3o
                                  // construtora (tratam-se de par\u00e2metros PUBLICOS)
                                  
    //comprimento = 12*texto.length();
    
    // dimens\u00f5es-padr\u00e3o
    altura = 20;    
    comprimento = 92;
  }

  //CONSTUTRUTORA EXTENDIDA (entramos com os valores de 'comprimento' e 'altura' do ret\u00e2ngulo interno do bot\u00e3o)
  Botao(float _xpos, float _ypos, float _comprimento, float _altura, String _texto){
    xpos = _xpos;
    ypos = _ypos;
    comprimento = _comprimento;
    altura = _altura;
    texto = _texto;
    situacao = "LIBERADO";        // da mesma forma que na outra construtora 
    psituacao = "SELECIONADO"; 
    }

  //desenhamos o bot\u00e3o
  public void display(){
    noStroke();
    pushMatrix();
    translate(xpos, ypos); //origem: centro do bot\u00e3o
   
    // as cores dos ret\u00e2ngulos dependem da situa\u00e7\u00e3o em que o bot\u00e3o se encontra
    
    //ret\u00e2ngulo externo
    if(situacao == "LIBERADO")
      fill(180);
    else if(situacao == "PRESSIONADO")
      fill(200);
    else if(situacao == "SELECIONADO")
      fill(220);
    
    rect(0, 0, comprimento + 20, altura + 20); // desenhamos o ret\u00e2ngulo
    
    //ret\u00e2ngulo intermedi\u00e1rio
    if(situacao == "LIBERADO")
      fill(200);
    else if(situacao == "PRESSIONADO")
      fill(220);
    else if(situacao == "SELECIONADO")
      fill(180);
    
    rect(0, 0, comprimento + 10, altura + 10); // desenhamos o ret\u00e2ngulo
    
    //ret\u00e2ngulo interno
    if(situacao == "LIBERADO")
      fill(220);
    else if(situacao == "PRESSIONADO")
      fill(180);
    else if(situacao == "SELECIONADO")
      fill(200);
    
    rect(0, 0, comprimento , altura); // desenhamos o ret\u00e2ngulo
    
    //a cor do texto tamb\u00e9m depende da situa\u00e7\u00e3o do bot\u00e3o
    if(situacao == "PRESSIONADO")
      fill(255);
    else
      fill(0);
    
    text(texto, 0, 4); // escrevemos o texto
    
    popMatrix();  
  }
  
  //fun\u00e7\u00e3o chamada pelo clique do bot\u00e3o (embora o nome seja o mesmo da fun\u00e7\u00e3o da biblioteca interna
  //                                     do Processing, esta fun\u00e7\u00e3o n\u00e3o \u00e9 chamada automaticamente
  //                                     pelo clique do mouse. Portanto devemos cham\u00e1-la dentro da
  //                                     fun\u00e7\u00e3o mousePressed do Processing)
  public void mousePressed(){
    float dx = mouseX - xpos;
    float dy = mouseY - ypos;
    
    //se, no momento do clique do mouse, este estiver sobre o bot\u00e3o, o mesmo assume a situa\u00e7\u00e3o "PRESSIONADO"
    if(abs(dx) < (comprimento)/2 && abs(dy) < (altura)/2)
      situacao = "PRESSIONADO";    
  }
  
  //fun\u00e7\u00e3o chamada pela libera\u00e7\u00e3o do bot\u00e3o do mouse (vale o mesmo dito para a fun\u00e7\u00e3o mousePressed)
  public void mouseReleased(){
    
    /*if(situacao == "PRESSIONADO" && psituacao == "LIBERADO"){
      situacao = psituacao;
      psituacao = "SELECIONADO";
    }
    
    if(situacao == "PRESSIONADO" && psituacao == "SELECIONADO"){
      situacao = psituacao;
      psituacao = "LIBERADO";
    }*/
    
    //se o bot\u00e3o foi pressionado, ap\u00f3s a libera\u00e7\u00e3o do mouse o primeiro assume a situa\u00e7\u00e3o "SELECIONADO"
    if(situacao == "PRESSIONADO"){
      situacao = "SELECIONADO";
      psituacao = "LIBERADO";
    }
      
  }
  
  //fun\u00e7\u00e3o que libera o bot\u00e3o
  public void libera(){
    psituacao = "SELECIONADO";
    situacao = "LIBERADO";
  }

}
class Credito{
  
  Botao botao;
  
  PFont f;
  
  boolean mais;
  
  Credito(){
    botao = new Botao(width - 40, 20, 45, 20, "Cr\u00e9ditos");
    f = loadFont("ArialUnicodeMS-11.vlw");
    mais = false;
  }

 public void display(){     
   textFont(f, 11);
   botao.display();
   rectMode(CORNERS);
   if( botao.situacao == "SELECIONADO" ){
     fill(240, 200);
     textAlign(LEFT);     
     if(mais){
       //rect(width/2, height/2, 400, 400);
       rect(width/2 - 200, height/2 - 200, width/2 + 200, height/2 + 200);
       fill(0);
       text( "\t\t                                                                            CR\u00c9DITOS" +
           "\n\nDesenvolvimento: Gustavo S. Jacomini (2\u00baano, Engenharia Civil-Escola Polit\u00e9cnica)" +
           "\n" + 
           "\n\nOrienta\u00e7\u00e3o: Ivette Frida Cymbaum Oppenheim (Departamento de Engenharia " + 
           "\n                                                                        Metal\u00fargica e de Materiais - Escola Polit\u00e9cnica)" +
           "\n\ncriado em:" + 
//           " Processing" + 
           "\n\n\nO meio:    'Processing' \u00e9 uma linguagem de programa\u00e7\u00e3o e ambiente de" +  
           "\n                      desenvolvimento com foco principal nas artes visuais, apro-" +
           "\n                      veitando-se da plataforma Java e de uma comunidade virtual" + 
           "\n                      intensamente atuante atrav\u00e9s do compartilhamento de c\u00f3di-" + 
           "\n                      gos e bibliotecas atrav\u00e9s da internet." + 
           "\n\n                      Surgiu como iniciativa de Casey Reas e Ben Fry, pesquisa-" +
           "\n                      dores do Instituto de Tecnologia de Massachussets (MIT)," +
           "\n                      sendo, desde sua cria\u00e7\u00e3o,disponibilizado gratuitamente co-" +
           "\n                      mo software livre,acess\u00edvel a qualquer interessado a partir" + 
           "\n                      do download no site 'processing.org',onde podem ser en-" +
           "\n                      contradas informa\u00e7\u00f5es adicionais sobre 'Processing'." +    
           "\n\n\nApoio:     'Anima\u00e7\u00f5es para Disciplinas Introdut\u00f3rias de Ci\u00eancia e Engenharia dos"  + 
           "\n                      Materiais' \u00e9 integrante do 'Programa Ensinar com Pesquisa', inciativa da"  +
           "\n                     Pr\u00f3-Reitoria de Gradua\u00e7\u00e3o da Universidade de S\u00e3o Paulo, \u00e0 qual expres-"  +
           "\n                     samos nossos agradecimentos (Projeto 598 -  Ano 2009).",width/2 - 190, height/2 - 170);
     }
     
     else if(mais == false){
       rect(width/2 - 200, height/2 - 200, width/2 + 200, height/2 + 30);
       fill(0);
       text( "\t\t                                                                            CR\u00c9DITOS" +
           "\n\nDesenvolvimento: Gustavo S. Jacomini (2\u00baano, Engenharia Civil-Escola Polit\u00e9cnica)" +
           "\n" + 
           "\n\nOrienta\u00e7\u00e3o: Ivette Frida Cymbaum Oppenheim (Departamento de Engenharia " + 
           "\n                                                                        Metal\u00fargica e de Materiais - Escola Polit\u00e9cnica)" +
           "\n\ncriado em:" +    
           "\n\n\nApoio:     'Anima\u00e7\u00f5es para Disciplinas Introdut\u00f3rias de Ci\u00eancia e Engenharia dos"  + 
           "\n                      Materiais' \u00e9 integrante do 'Programa Ensinar com Pesquisa', inciativa da"  +
           "\n                     Pr\u00f3-Reitoria de Gradua\u00e7\u00e3o da Universidade de S\u00e3o Paulo, \u00e0 qual expres-"  +
           "\n                     samos nossos agradecimentos (Projeto 598 -  Ano 2009).",width/2 - 190, height/2 - 170);
       
     }
     textAlign(CENTER);
     
     if(!mais)
       fill(0, 0, 150);
     else if(mais)
       fill(150);
     
     if(abs(mouseX - (width/2 - 86)) < 30 && abs(mouseY - (height/2 - 68)) < 10){
       if(mousePressed)
         fill(255);
       else{
         if(!mais)
           fill(100, 100, 255);
         else if(mais)
           fill(200);
       }
     }
     
     text("Processing", width/2 - 86, height/2 - 68);
    
          
   }
   rectMode(CENTER);
 }
 
 class Botao{
  float xpos, ypos; // coordenadas do centro do bot\u00e3o
  
  String texto, // guarda o texto a ser impresso no bot\u00e3o
  
         situacao, // guarda a situa\u00e7\u00e3o em que se encontra o bot\u00e3o, podendo estar "LIBERADO", "PRESSIONADO" ou "SELECIONADO"
  
         psituacao;  // guarda a situa\u00e7\u00e3o 'est\u00e1vel' que o bot\u00e3o assumira antes de ser pressionado pela \u00faltima vez
                     // (pode guardar "LIBERADO" ou "SELECIONADO") 
  
  float comprimento; // comprimento de refer\u00eancia do bot\u00e3o (ret\u00e2ngulo interno)
  
  float altura; // altura de refer\u00eancia do bot\u00e3o (ret\u00e2ngulo interno)
  
  
  //CONSTRUTORA SIMPLES//
  Botao(float _xpos, float _ypos, String _texto){
    xpos = _xpos;
    ypos = _ypos;
    texto = _texto;
    situacao = "LIBERADO";        // a priore, o bot\u00e3o \u00e9 iniciado como se n\u00e3o estivesse ativado 
    psituacao = "SELECIONADO";    // caso queiramos que ele apresente-se 'selecionado' logo de
                                  // in\u00edcio, trocamos estas vari\u00e1veis apos a chamada da fun\u00e7\u00e3o
                                  // construtora (tratam-se de par\u00e2metros PUBLICOS)
                                  
    //comprimento = 12*texto.length();
    
    // dimens\u00f5es-padr\u00e3o
    altura = 20;    
    comprimento = 92;
  }

  //CONSTUTRUTORA EXTENDIDA (entramos com os valores de 'comprimento' e 'altura' do ret\u00e2ngulo interno do bot\u00e3o)
  Botao(float _xpos, float _ypos, float _comprimento, float _altura, String _texto){
    xpos = _xpos;
    ypos = _ypos;
    comprimento = _comprimento;
    altura = _altura;
    texto = _texto;
    situacao = "LIBERADO";        // da mesma forma que na outra construtora 
    psituacao = "SELECIONADO"; 
    }

  //desenhamos o bot\u00e3o
  public void display(){
    noStroke();
    rectMode(CENTER);
    pushMatrix();
    translate(xpos, ypos); //origem: centro do bot\u00e3o
   
    // as cores dos ret\u00e2ngulos dependem da situa\u00e7\u00e3o em que o bot\u00e3o se encontra
    
   /* //ret\u00e2ngulo externo
    if(situacao == "LIBERADO")
      fill(255 - 180);
    else if(situacao == "PRESSIONADO")
      fill(255 - 200);
    else if(situacao == "SELECIONADO")
      fill(255 - 220);
    
    rect(0, 0, comprimento + 20, altura + 20); // desenhamos o ret\u00e2ngulo
    */
    //ret\u00e2ngulo intermedi\u00e1rio
    if(situacao == "LIBERADO")
      fill(200);
    else if(situacao == "PRESSIONADO")
      fill(220);
    else if(situacao == "SELECIONADO")
      fill(180);
    
    rect(0, 0, comprimento + 10, altura + 10); // desenhamos o ret\u00e2ngulo
    
    //ret\u00e2ngulo interno
    if(situacao == "LIBERADO")
      fill(220);
    else if(situacao == "PRESSIONADO")
      fill(180);
    else if(situacao == "SELECIONADO")
      fill(200);
    
    rect(0, 0, comprimento , altura); // desenhamos o ret\u00e2ngulo
    
    //a cor do texto tamb\u00e9m depende da situa\u00e7\u00e3o do bot\u00e3o
    if(situacao == "PRESSIONADO")
      fill(255);
    else
      fill(0);
    
    text(texto, 0, 4); // escrevemos o texto
    
    popMatrix();  
  }
  
  //fun\u00e7\u00e3o chamada pelo clique do bot\u00e3o (embora o nome seja o mesmo da fun\u00e7\u00e3o da biblioteca interna
  //                                     do Processing, esta fun\u00e7\u00e3o n\u00e3o \u00e9 chamada automaticamente
  //                                     pelo clique do mouse. Portanto devemos cham\u00e1-la dentro da
  //                                     fun\u00e7\u00e3o mousePressed do Processing)
  public void mousePressed(){
    float dx = mouseX - xpos;
    float dy = mouseY - ypos;
    
    //se, no momento do clique do mouse, este estiver sobre o bot\u00e3o, o mesmo assume a situa\u00e7\u00e3o "PRESSIONADO"
    if(abs(dx) < (comprimento)/2 && abs(dy) < (altura)/2)
      situacao = "PRESSIONADO";
    
    //gerenciamos a apresenta\u00e7\u00e3o de informa\u00e7\u00f5es adicionais pelo clique sobre a palavra 'Processing'
    if(abs(mouseX - (width/2 - 86)) < 30 && abs(mouseY - (height/2 - 68)) < 10)
      mais = !mais;    
  }
  
  //fun\u00e7\u00e3o chamada pela libera\u00e7\u00e3o do bot\u00e3o do mouse (vale o mesmo dito para a fun\u00e7\u00e3o mousePressed)
  public void mouseReleased(){
    
    /*if(situacao == "PRESSIONADO" && psituacao == "LIBERADO"){
      situacao = psituacao;
      psituacao = "SELECIONADO";
    }
    
    if(situacao == "PRESSIONADO" && psituacao == "SELECIONADO"){
      situacao = psituacao;
      psituacao = "LIBERADO";
    }*/
    
    //se o bot\u00e3o foi pressionado, ap\u00f3s a libera\u00e7\u00e3o do mouse o primeiro assume a situa\u00e7\u00e3o "SELECIONADO"
    if(situacao == "PRESSIONADO" && psituacao == "SELECIONADO"){
      situacao = "SELECIONADO";
      psituacao = "LIBERADO";
    }
    
    if(situacao == "PRESSIONADO" && psituacao == "LIBERADO"){
      situacao = "LIBERADO";
      psituacao = "SELECIONADO";
    }
    
    
       
  }
  
  //fun\u00e7\u00e3o que libera o bot\u00e3o
  public void libera(){
    psituacao = "SELECIONADO";
    situacao = "LIBERADO";
  }

}
}
 
class Grafico{
  PVector o; // vetor posi\u00e7\u00e3o da origem do gr\u00e1fico
  float altura; // altura total do gr\u00e1fico
  float largura; // largura total do gr\u00e1fico
  float x_max, x_min; // abscissas ABSOLUTAS maxima e m\u00ednima
  float y_max, y_min; // ordenadas ABSOLUTAS m\u00e1xima e m\u00ednima
  float escala_x, escala_y; // escalas para que a dist\u00e2ncias entre os dados ABSOLUTOS se encaixem na largura e na altura
  float rmin = 10; // dist\u00e2ncia de liga\u00e7\u00e3o entre os \u00e1tomos (em \u00e2ngstrons)
  
  //CONSTRUTORA
  Grafico(PVector origem, float _altura, float _largura, float _x_max, float _x_min, float _y_max, float _y_min){
    o = new PVector (origem.x, origem.y);
    altura = _altura;
    largura = _largura;
    x_max = _x_max;
    x_min = _x_min;
    y_max = _y_max;
    y_min = _y_min;   
    
    //definimos as escalas
    escala_x = largura/(x_max - x_min);
    escala_y = altura/(y_max - y_min);
  }  
  
  // fun\u00e7\u00e3o que desenha o gr\u00e1fico
  public void desenha_grafico(){
    strokeWeight(2); 
    stroke(200);// os eixos ordenados s\u00e3o claros, destacando a linha da fun\u00e7\u00e3o
     
     // desenhamos os eixos ordenados
     pushMatrix();
     translate(o.x, o.y);
     line((x_min)*(escala_x) - 5, 0, (x_max)*(escala_x), 0);
     line((x_max)*(escala_x), 0, (x_max)*(escala_x) - 5, 5);
     line((x_max)*(escala_x), 0, (x_max)*(escala_x) - 5, -5);    
     line(0, (y_min)*(escala_y), 0, (y_max)*(escala_y));
     line(0, (y_min)*(escala_y), 5, (y_min)*(escala_y) + 5);
     line(0, (y_min)*(escala_y), -5, (y_min)*(escala_y) + 5);
     // nomeamos os eixos
     textFont(f, 10);
     fill(150);
     pushMatrix();
     translate((x_max)*(escala_x) - 20, 20);
     text("SEPARA\u00c7\u00c3O INTERAT\u00d4MICA", 0, 0);
     popMatrix();
     pushMatrix();
     translate(-20, -20);
     rotate(3*PI/2);
     if(modo == true)
       text("ENERGIA POTENCIAL", 0, 0);
     if(modo == false)
       text("FOR\u00c7A", 0, 0);
     popMatrix();
     popMatrix(); 
     
     // para desenhar o gr\u00e1fico da fun\u00e7\u00e3o, pegamos duas coordenadas (x, y) 
     // chamadas 'a' e 'b' e tra\u00e7amos uma linha entre elas     
     strokeWeight(1);
     stroke(0);
     for(float t = x_min + 0.02f; t < x_max - 1; t += 0.02f){ // 't' \u00e9 o par\u00e2metro no qual calculamos a fun\u00e7\u00e3o
      float p = rmin/t;
      float ax = t*escala_x; // criamos 'a'
      float ay = funcao(p);
      float t1 = t + 0.02f;
      p = rmin/t1;
      float bx = t1*escala_x; // criamos 'b'
      float by = funcao(p);
      if((ay < y_max*escala_y && ay > y_min*escala_y) && (by < y_max*escala_y && by > y_min*escala_y) ){
        pushMatrix();
        translate(o.x, o.y); // mudan\u00e7a de origem
        line(ax, ay, bx, by); // linha entre os pontos 'a' e 'b'
        popMatrix();
      }
     }
     noStroke();
  }
  
  /*void desenha_grafico(PGraphics pg){
     pg.stroke(255);
     
     pg.pushMatrix();
     pg.translate(o.x, o.y);
     pg.line((x_min)*(escala_x), 0, (x_max)*(escala_x), 0);
     pg.line(0, y_min*escala_y, 0, y_max*escala_y);     
     for(float t = x_min + 0.1; t < x_max - 0.1; t += 0.1){
      float rmin = 3.5;
      float p = rmin/t;
      float ax = t*escala_x;
      float ay = 100*(pow(p, 12) - pow(p, 6))*escala_y;
      float t1 = t + 0.1;
      p = rmin/t1;
      float bx = t1*escala_x;
      float by = 100*(pow(p, 12) - pow(p, 6))*escala_y;
      if((ay < y_max && ay > y_min) && (by < y_max && by > y_min) ){
        pg.strokeWeight(2);
        pg.line(ax, ay, bx, by);
      }
     pg.popMatrix();
     }
  }*/
  
  //calculamos as fun\u00e7\u00f5es
  public float funcao(float x){
    float y = 0;
    // dependendo do modo, a fun\u00e7\u00e3o devolve o valor (em escala) da energia ou da for\u00e7a
    if(modo)
      y = (- 100*(pow(x, 8) - 2*pow(x, 4))*escala_y); //fun\u00e7\u00e3o da energia (o sinal negativo refere-se \u00e0 troca de sentido dos eixos no processing)
    if(!modo)
      y = (800*(pow(x, 7) - pow(x, 3))*escala_y); //fun\u00e7\u00e3o da for\u00e7a
    return y;
  }
} 
  static public void main(String args[]) {
    PApplet.main(new String[] { "--bgcolor=#FFFFFF", "Distancia_Interatomica" });
  }
}
